@@ -171,7 +171,7 @@ class expose(object):
         metadict['func_name'] = func.__name__
         metadict['exposed'] = True
         metadict['hide'] = self.hide
-        metadict['help'] = self.help
+        metadict['help'] = self.help or func.__doc__
         metadict['aliases'] = self.aliases
         metadict['aliases_only'] = self.aliases_only
         metadict['controller'] = None  # added by the controller
@@ -365,7 +365,7 @@ class CementBaseController(handler.CementBaseHandler):
                 commands.append(func)
 
         # process stacked controllers second for commands and args
-        for contr in handler.list('controller'):
+        for contr in self.app.handler.list('controller'):
             # don't include self here
             if contr == self.__class__:
                 continue
@@ -428,7 +428,6 @@ class CementBaseController(handler.CementBaseHandler):
         self._visible_commands.sort()
 
     def _get_dispatch_command(self):
-        default_func = self._meta.default_func
         default_func_key = re.sub('_', '-', self._meta.default_func)
 
         if (len(self.app.argv) <= 0) or (self.app.argv[0].startswith('-')):
