@@ -1,25 +1,31 @@
 """Cement core foundation module."""
 
-import re
 import os
 import sys
 import signal
-import copy
 import platform
 from time import sleep
-from ..core import backend, exc, log, config, plugin, interface
+from ..core import backend, exc, log, config, plugin
 from ..core import output, extension, arg, controller, meta, cache, mail
 from ..core.handler import HandlerManager
 from ..core.hook import HookManager
 from ..utils.misc import is_true, minimal_logger
 from ..utils import fs
 
-if sys.version_info[0] >= 3:
-    from imp import reload  # pragma: nocover
+# The `imp` module is deprecated in favor of `importlib` in 3.4, but it
+# wasn't introduced until 3.1.  Finally, reload is a builtin on Python < 3
+pyver = sys.version_info
+if pyver[0] >= 3 and pyver[1] >= 4:                # pragma: nocover  # noqa
+    from importlib import reload as reload_module  # pragma: nocover  # noqa
+elif pyver[0] >= 3:                                # pragma: nocover  # noqa
+    from imp import reload as reload_module        # pragma: nocover  # noqa
+else:                                              # pragma: nocover  # noqa
+    reload_module = reload                         # pragma: nocover  # noqa
+
 
 LOG = minimal_logger(__name__)
 if platform.system() == 'Windows':
-    SIGNALS = [signal.SIGTERM, signal.SIGINT]
+    SIGNALS = [signal.SIGTERM, signal.SIGINT]   # pragma: nocover
 else:
     SIGNALS = [signal.SIGTERM, signal.SIGINT, signal.SIGHUP]
 
@@ -36,7 +42,7 @@ def add_handler_override_options(app):
         return
 
     for i in app._meta.handler_override_options:
-        if i not in interface.list():
+        if i not in app.handler.list_types():
             LOG.debug("interface '%s'" % i +
                       " is not defined, can not override handlers")
             continue
@@ -204,6 +210,11 @@ class CementApp(meta.MetaMixin):
         ``sys.exit(X)`` where ``X`` is ``self.exit_code``.
         """
 
+        config_extension = '.conf'
+        """
+        Extension used to identify application and plugin configuration files.
+        """
+
         config_files = None
         """
         List of config files to parse.
@@ -222,6 +233,9 @@ class CementApp(meta.MetaMixin):
         Files are loaded in order, and have precedence in order.  Therefore,
         the last configuration loaded has precedence (and overwrites settings
         loaded from previous configuration files).
+
+        Note that ``.conf`` is the default config file extension, defined by
+        ``CementApp.Meta.config_extension``.
         """
 
         plugins = []
@@ -234,7 +248,8 @@ class CementApp(meta.MetaMixin):
         plugin_config_dirs = None
         """
         A list of directory paths where plugin config files can be found.
-        Files must end in `.conf` or they will be ignored.
+        Files must end in ``.conf`` (or the extension defined by
+        ``CementApp.Meta.config_extension``) or they will be ignored.
 
         Note: Though ``CementApp.Meta.plugin_config_dirs`` is ``None``, Cement
         will set this to a default list based on ``CementApp.Meta.label``.
@@ -252,9 +267,11 @@ class CementApp(meta.MetaMixin):
 
         plugin_config_dir = None
         """
-        A directory path where plugin config files can be found.  Files
-        must end in `.conf`.  By default, this setting is also overridden
-        by the ``[<app_label>] -> plugin_config_dir`` config setting parsed in
+        A directory path where plugin config files can be found.  Files must
+        end in ``.conf`` (or the extension defined by
+        ``CementApp.Meta.config_extension``) or they will be ignored.  By
+        default, this setting is also overridden by the
+        ``[<app_label>] -> plugin_config_dir`` config setting parsed in
         any of the application configuration files.
 
         If set, this item will be **appended** to
@@ -399,6 +416,32 @@ class CementApp(meta.MetaMixin):
         config_defaults = None
         """Default configuration dictionary.  Must be of type 'dict'."""
 
+        meta_defaults = {}
+        """
+        Default metadata dictionary used to pass high level options from the
+        application down to handlers at the point they are registered by the
+        framework **if the handler has not already been instantiated**.
+
+        For example, if requiring the ``json`` extension, you might want to
+        override ``JsonOutputHandler.Meta.json_module`` with ``ujson`` by
+        doing the following
+
+        .. code-block:: python
+
+            from cement.core.foundation import CementApp
+            from cement.utils.misc import init_defaults
+
+            META = init_defaults('output.json')
+            META['output.json']['json_module'] = 'ujson'
+
+            class MyApp(CementApp):
+                class Meta:
+                    label = 'myapp'
+                    extensions = ['json']
+                    meta_defaults = META
+
+        """
+
         catch_signals = SIGNALS
         """
         List of signals to catch, and raise exc.CaughtSignal for.
@@ -634,6 +677,41 @@ class CementApp(meta.MetaMixin):
         has changed or exists.
         """
 
+        alternative_module_mapping = {}
+        """
+        EXPERIMENTAL FEATURE: This is an experimental feature added in Cement
+        2.9.x and may or may not be removed in future versions of Cement.
+
+        Dictionary of alternative, **drop-in** replacement modules to use
+        selectively throughout the application, framework, or
+        extensions.  Developers can optionally use the
+        ``CementApp.__import__()`` method to import simple modules, and if
+        that module exists in this mapping it will import the alternative
+        library in it's place.
+
+        This is a low-level feature, and may not produce the results you are
+        expecting.  It's purpose is to allow the developer to replace specific
+        modules at a high level.  Example: For an application wanting to use
+        ``ujson`` in place of ``json``, the developer could set the following:
+
+        .. code-block:: python
+
+            alternative_module_mapping = {
+                'json' : 'ujson',
+            }
+
+        In the app, you would then load ``json`` as:
+
+        .. code-block:: python
+
+            _json = app.__import__('json')
+            _json.dumps(data)
+
+
+        Obviously, the replacement module **must be** a drop-in replace and
+        function the same.
+        """
+
     def __init__(self, label=None, **kw):
         super(CementApp, self).__init__(**kw)
 
@@ -651,8 +729,10 @@ class CementApp(meta.MetaMixin):
         self._loaded_bootstrap = None
         self._parsed_args = None
         self._last_rendered = None
+        self._extended_members = []
         self.__saved_stdout__ = None
         self.__saved_stderr__ = None
+        self.__retry_hooks__ = []
         self.handler = None
         self.hook = None
 
@@ -714,6 +794,8 @@ class CementApp(meta.MetaMixin):
         LOG.debug("extending appication with '.%s' (%s)" %
                   (member_name, member_object))
         setattr(self, member_name, member_object)
+        if member_name not in self._extended_members:
+            self._extended_members.append(member_name)
 
     def _validate_label(self):
         if not self._meta.label:
@@ -756,7 +838,7 @@ class CementApp(meta.MetaMixin):
 
                 self._loaded_bootstrap = sys.modules[self._meta.bootstrap]
             else:
-                reload(self._loaded_bootstrap)
+                reload_module(self._loaded_bootstrap)
 
         for res in self.hook.run('pre_setup', self):
             pass
@@ -772,6 +854,9 @@ class CementApp(meta.MetaMixin):
         self._setup_output_handler()
         self._setup_controllers()
 
+        for hook_spec in self.__retry_hooks__:
+            self.hook.register(*hook_spec)
+
         for res in self.hook.run('post_setup', self):
             pass
 
@@ -843,6 +928,9 @@ class CementApp(meta.MetaMixin):
         :returns: ``None``
         """
         LOG.debug('reloading the %s application' % self._meta.label)
+        for member in self._extended_members:
+            delattr(self, member)
+        self._extended_members = []
         self.handler.__handlers__ = {}
         self.hook.__hooks__ = {}
         self._lay_cement()
@@ -931,11 +1019,11 @@ class CementApp(meta.MetaMixin):
 
         """
         if not is_true(self._meta.ignore_deprecation_warnings):
-            self.log.warn("Cement Deprecation Warning: " +
-                          "CementApp.get_last_rendered() has been " +
-                          "deprecated, and will be removed in future " +
-                          "versions of Cement.  You should use the " +
-                          "CementApp.last_rendered property instead.")
+            self.log.warning("Cement Deprecation Warning: " +
+                             "CementApp.get_last_rendered() has been " +
+                             "deprecated, and will be removed in future " +
+                             "versions of Cement.  You should use the " +
+                             "CementApp.last_rendered property instead.")
         return self._last_rendered
 
     @property
@@ -987,7 +1075,7 @@ class CementApp(meta.MetaMixin):
             self._suppress_output()
 
         # Forward/Backward compat, see Issue #311
-        if self._meta.use_backend_globals:
+        if self._meta.use_backend_globals is True:
             backend.__hooks__ = {}
             backend.__handlers__ = {}
             self.handler = HandlerManager(use_backend_globals=True)
@@ -1014,14 +1102,24 @@ class CementApp(meta.MetaMixin):
             self.hook.define(label)
 
         # register some built-in framework hooks
-        self.hook.register(
-            'post_setup', add_handler_override_options, weight=-99)
+        self.hook.register('post_setup', add_handler_override_options,
+                           weight=-99)
         self.hook.register('post_argument_parsing',
                            handler_override, weight=-99)
 
-        # register application hooks from meta
-        for label, func in self._meta.hooks:
-            self.hook.register(label, func)
+        # register application hooks from meta.  the hooks listed in
+        # CementApp.Meta.hooks are registered here, so obviously can not be
+        # for any hooks other than the builtin framework hooks that we just
+        # defined here (above).  Anything that we couldn't register here
+        # will be retried after setup
+        self.__retry_hooks__ = []
+        for hook_spec in self._meta.hooks:
+            if not self.hook.defined(hook_spec[0]):
+                LOG.debug('hook %s not defined, will retry after setup' %
+                          hook_spec[0])
+                self.__retry_hooks__.append(hook_spec)
+            else:
+                self.hook.register(*hook_spec)
 
         # define and register handlers
         self.handler.define(extension.IExtension)
@@ -1098,7 +1196,17 @@ class CementApp(meta.MetaMixin):
             self.catch_signal(signum)
 
     def _resolve_handler(self, handler_type, handler_def, raise_error=True):
-        han = self.handler.resolve(handler_type, handler_def, raise_error)
+        meta_defaults = {}
+        if type(handler_def) == str:
+            _meta_label = "%s.%s" % (handler_type, handler_def)
+            meta_defaults = self._meta.meta_defaults.get(_meta_label, {})
+        elif hasattr(handler_def, 'Meta'):
+            _meta_label = "%s.%s" % (handler_type, handler_def.Meta.label)
+            meta_defaults = self._meta.meta_defaults.get(_meta_label, {})
+
+        han = self.handler.resolve(handler_type, handler_def,
+                                   raise_error=raise_error,
+                                   meta_defaults=meta_defaults)
         if han is not None:
             han._setup(self)
             return han
@@ -1123,10 +1231,11 @@ class CementApp(meta.MetaMixin):
 
         if self._meta.config_files is None:
             label = self._meta.label
+            ext = self._meta.config_extension
 
             self._meta.config_files = [
-                os.path.join('/', 'etc', label, '%s.conf' % label),
-                os.path.join(fs.HOME_DIR, '.%s.conf' % label),
+                os.path.join('/', 'etc', label, '%s%s' % (label, ext)),
+                os.path.join(fs.HOME_DIR, '.%s%s' % (label, ext)),
                 os.path.join(fs.HOME_DIR, '.%s' % label, 'config'),
             ]
 
@@ -1238,10 +1347,14 @@ class CementApp(meta.MetaMixin):
 
         # template dirs
         if self._meta.template_dirs is None:
-            self._meta.template_dirs = [
+            self._meta.template_dirs = []
+            paths = [
                 os.path.join(fs.HOME_DIR, '.%s' % label, 'templates'),
                 '/usr/lib/%s/templates' % label,
             ]
+            for path in paths:
+                self.add_template_dir(path)
+
         template_dir = self._meta.template_dir
         if template_dir is not None:
             if template_dir not in self._meta.template_dirs:
@@ -1327,6 +1440,56 @@ class CementApp(meta.MetaMixin):
         """
         pass
 
+    def add_template_dir(self, path):
+        """
+        Append a directory path to the list of template directories to parse
+        for templates.
+
+        :param path: Directory path that contains template files.
+
+        Usage:
+
+        .. code-block:: python
+
+            app.add_template_dir('/path/to/my/templates')
+
+        """
+        path = fs.abspath(path)
+        if path not in self._meta.template_dirs:
+            self._meta.template_dirs.append(path)
+
+    def remove_template_dir(self, path):
+        """
+        Remove a directory path from the list of template directories to parse
+        for templates.
+
+        :param path: Directory path that contains template files.
+
+        Usage:
+
+        .. code-block:: python
+
+            app.remove_template_dir('/path/to/my/templates')
+
+        """
+        path = fs.abspath(path)
+        if path in self._meta.template_dirs:
+            self._meta.template_dirs.remove(path)
+
+    def __import__(self, obj, from_module=None):
+        # EXPERIMENTAL == UNDOCUMENTED
+        mapping = self._meta.alternative_module_mapping
+
+        if from_module is not None:
+            _from = mapping.get(from_module, from_module)
+            _loaded = __import__(_from, globals(), locals(), [obj], 0)
+            return getattr(_loaded, obj)
+        else:
+            obj = mapping.get(obj, obj)
+            _loaded = __import__(obj, globals(), locals(), [], 0)
+
+        return _loaded
+
     def __enter__(self):
         self.setup()
         return self
