@@ -285,37 +285,43 @@ class ArgparseExtTestCase(test.CementExtTestCase):
             app._meta.argv = ['cmd2']
             res = app.run()
             self.eq(res, "Inside Second.cmd2")
-
+        self.tearDown()
+        
         self.setUp()
         with self.app as app:
             app._meta.argv = ['third', 'cmd3']
             res = app.run()
             self.eq(res, "Inside Third.cmd3")
+        self.tearDown()
 
         self.setUp()
         with self.app as app:
             app._meta.argv = ['third', 'cmd4']
             res = app.run()
             self.eq(res, "Inside Fourth.cmd4")
+        self.tearDown()
 
         self.setUp()
         with self.app as app:
             app._meta.argv = ['third', 'fifth', 'cmd5']
             res = app.run()
             self.eq(res, "Inside Fifth.cmd5")
+        self.tearDown()
 
         self.setUp()
         with self.app as app:
             app._meta.argv = ['third', 'fifth', 'sixth', 'cmd6']
             res = app.run()
             self.eq(res, "Inside Sixth.cmd6")
+        self.tearDown()
 
         self.setUp()
         with self.app as app:
             app._meta.argv = ['third', 'cmd7']
             res = app.run()
             self.eq(res, "Inside Seventh.cmd7")
-
+        self.tearDown()
+        
     def test_base_cmd1_parsing(self):
         with self.app as app:
             app._meta.argv = ['--foo=bar', 'cmd1']
@@ -597,3 +603,21 @@ class ArgparseExtTestCase(test.CementExtTestCase):
             app._setup_arg_handler()
             res = app.run()
             self.eq(res, "Inside Aliases.aliases_cmd1")
+
+    def test_unknown_arguments(self):
+        self.reset_backend()
+
+        class MyArgumentHandler(ArgparseArgumentHandler):
+            class Meta:
+                label = 'my_argument_handler'
+                ignore_unknown_arguments = True
+        self.app = self.make_app(APP,
+                                 argument_handler=MyArgumentHandler,
+                                 )
+        with self.app as app:
+            app._meta.argv = ['-l', 'some-other-argument']
+            app.run()
+            res = '-l' in app.args.unknown_args
+            self.ok(res)
+            res = 'some-other-argument' in app.args.unknown_args
+            self.ok(res)
